#pragma once

#include "Triangle.h"

#include <vector>

/**
 * @brief Defines the data structure for a node in a Binary Space Partition (BSP) tree.
 */
struct BspNode {
    BspNode( const std::vector<Triangle> &triangles, const bool &split=true );
    virtual ~BspNode();
    void gather( std::vector<Triangle> &sortedTriangles,  const glm::vec3 &eyePosition );

    /** This node's triangle. */
    Triangle m_triangle;

    /** This node's plane, defined by the triangle. */
    glm::vec3 m_planeNormal;

    /** The subtree of this node, containing all remaining triangles that are in front. */
    BspNode *m_front = nullptr;

    /** The subtree of this node, containing all remaining triangles that are behind. */
    BspNode *m_back = nullptr;
    
private:
    bool splitTriangle( const Triangle& tri, const glm::vec3& p, const glm::vec3& n, std::vector<Triangle>& newTriangles );
    glm::vec3 intersectPlaneLine( const glm::vec3 &p, const glm::vec3 &n, const glm::vec3 &v1, const glm::vec3 &v2 );
};

static glm::vec3 CreateNormal( glm::vec3 v0, glm::vec3 v1, glm::vec3 v2 );